@@ -11,12 +11,10 @@ package Geo::ReadGRIB;
 
 use 5.006_001;
 use strict;
-use warnings;
 use IO::File;
 use Carp;
 
-our $VERSION = 1.35;
-
+our $VERSION = 1.1;
 use Geo::ReadGRIB::PlaceIterator;
 
 my $LIB_DIR = "./";
@@ -60,7 +58,6 @@ sub new {
 
     $self->{fileName} = $gFile;
     $self->{DEBUG}    = 0;
-    $self->backflip( 0 );
 
     $self->openGrib();
 
@@ -115,10 +112,16 @@ sub openGrib {
     }
 
     foreach ( sort keys %head ) {
+
         #     print " $_: $head{$_}\n";
         $self->{$_} = $head{$_};
     }
 
+    foreach ( keys %head ) {
+
+        #     $self->{$_} = $head{$_};
+    }
+
 
     # reduce date string to 'time' format
     my ( $yr, $mo, $day, $hr ) = unpack 'A4A2A2A2', $self->{d};
@@ -184,26 +187,13 @@ sub _getCatalog {
         croak "Error in \$cmd: $?";
     }
 
-    my $timeRange = $self->TR;
-
-    my ( @line, $offset );
+    my @line;
     foreach (@cat) {
         @line = split /:/;
         $line[8] =~ s/P1=//;
-        $line[9] =~ s/P2=//;
-        if ( $timeRange == 0 ) {
-            $offset = $line[8];
-        }
-        elsif ( $timeRange == 10 ) {
-            $offset = ($line[8] * 256) + $line[9];
-        }
-        else {
-            croak "Time Range flag $timeRange not yet supported. Please ask the Geo::ReadGRIB maintainer to add support for GRIB files like this";
-        }
-
-        $offset = $offset * 3600 + $self->{TIME};
-        $self->{LAST_TIME} = $offset if $offset > $self->{LAST_TIME};
-        $self->{catalog}->{ $offset }->{ $line[3] } = $line[0];
+        $line[8] = $line[8] * 3600 + $self->{TIME};
+        $self->{LAST_TIME} = $line[8] if $line[8] > $self->{LAST_TIME};
+        $self->{catalog}->{ $line[8] }->{ $line[3] } = $line[0];
     }
 
     return;
@@ -319,100 +309,121 @@ sub parseGDS {
 }
 
 #--------------------------------------------------------------------------
-# sn_scan_flag( [flag] ) - getter/setter
+# sn_scan_flag( [flag] )
+# Getter/setter for flag
 #--------------------------------------------------------------------------
 sub sn_scan_flag {
-    my ( $self, $flag ) = @_;
+    my $self = shift;
+    my $flag = shift;
+
     $self->{SN_SCAN_FLAG} = $flag if defined $flag;
     return  $self->{SN_SCAN_FLAG};
 }
 
 #--------------------------------------------------------------------------
-# error( [error message] ) - getter/setter for errors
+# error( [error message] )
+# Getter/setter for errors
 #--------------------------------------------------------------------------
 sub error {
-    my ( $self, $error ) = @_;
+    my $self = shift;
+    my $error = shift;
+
     $self->{ERROR} = $error if defined $error;
     return  $self->{ERROR};
 }
 
 #--------------------------------------------------------------------------
-# Lo1( [val] ) - Getter/setter for Lo1
+# Lo1( [val] )
+# Getter/setter for Lo1
 #--------------------------------------------------------------------------
 sub Lo1 {
-    my ( $self, $flag ) = @_;
+    my $self = shift;
+    my $flag = shift;
+
     $self->{Lo1} = $flag if defined $flag;
     return  $self->{Lo1};
 }
 
 #--------------------------------------------------------------------------
-# Lo2( [flag] ) - Getter/setter for Lo2
+# Lo2( [flag] )
+# Getter/setter for Lo2
 #--------------------------------------------------------------------------
 sub Lo2 {
-    my ( $self, $flag ) = @_;
+    my $self = shift;
+    my $flag = shift;
+
     $self->{Lo2} = $flag if defined $flag;
     return  $self->{Lo2};
 }
 
 #--------------------------------------------------------------------------
-# LoInc( [flag] ) - Getter/setter for LoInc
+# LoInc( [flag] )
+# Getter/setter for LoInc
 #--------------------------------------------------------------------------
 sub LoInc {
-    my ( $self, $flag ) = @_;
+    my $self = shift;
+    my $flag = shift;
+
     $self->{LoInc} = $flag if defined $flag;
     return  $self->{LoInc};
 }
 
 #--------------------------------------------------------------------------
-# La1( [flag] ) - Getter/setter for La1
+# La1( [flag] )
+# Getter/setter for La1
 #--------------------------------------------------------------------------
 sub La1 {
-    my ( $self, $flag ) = @_;
+    my $self = shift;
+    my $flag = shift;
+
     $self->{La1} = $flag if defined $flag;
     return  $self->{La1};
 }
 
 #--------------------------------------------------------------------------
-# TR( [flag] ) - Getter/setter for TR
-#--------------------------------------------------------------------------
-sub TR {
-    my ( $self, $flag ) = @_;
-    $self->{TR} = $flag if defined $flag;
-    return  $self->{TR};
-}
-
-#--------------------------------------------------------------------------
-# La2( [flag] ) - Getter/setter for La2
+# La2( [flag] )
+# Getter/setter for La2
 #--------------------------------------------------------------------------
 sub La2 {
-    my ( $self, $flag ) = @_;
+    my $self = shift;
+    my $flag = shift;
+
     $self->{La2} = $flag if defined $flag;
     return  $self->{La2};
 }
 
 #--------------------------------------------------------------------------
-# LaInc( [flag] )-  Getter/setter for LaInc
+# LaInc( [flag] )
+# Getter/setter for LaInc
 #--------------------------------------------------------------------------
 sub LaInc {
-    my ( $self, $flag ) = @_;
+    my $self = shift;
+    my $flag = shift;
+
     $self->{LaInc} = $flag if defined $flag;
     return  $self->{LaInc};
 }
 
 #--------------------------------------------------------------------------
-# Ni( [flag] ) - Getter/setter for Ni
+# Ni( [flag] )
+# Getter/setter for Ni
 #--------------------------------------------------------------------------
 sub Ni {
-    my ( $self, $flag ) = @_;
+    my $self = shift;
+    my $flag = shift;
+
     $self->{Ni} = $flag if defined $flag;
     return  $self->{Ni};
 }
 
 #--------------------------------------------------------------------------
-# Nj( [flag] ) - Getter/setter for Nj
+# Nj( [flag] )
+# Getter/setter for Nj
 #--------------------------------------------------------------------------
 sub Nj {
-    my ( $self, $flag ) = @_;
+    my $self = shift;
+    my $flag = shift;
+
     $self->{Nj} = $flag if defined $flag;
     return  $self->{Nj};
 }
@@ -427,31 +438,6 @@ sub clearError {
 }
 
 
-#--------------------------------------------------------------------------
-# validLa( lat )
-#
-# check if lats are in range and return true if they arr, else false
-#--------------------------------------------------------------------------
-sub validLa {
-
-    my $self = shift;
-    my $lat  = shift;
-
-    if  ( $self->sn_scan_flag ) {
-        if ( $lat < $self->La1 or $lat > $self->La2 ) {
-            return 0;
-        }
-    }
-    else {
-        if ( $lat > $self->La1 or $lat < $self->La2 ) {
-            return 0;
-        }
-    }
-
-
-    return 1;
-}
-
 #--------------------------------------------------------------------------
 # validLo( 1ong )
 #
@@ -462,21 +448,14 @@ sub validLo {
     my $self = shift;
     my $lo   = shift;
 
-    if  ( $self->sn_scan_flag ) {
-        $lo += $self->{Lo_SHIFT};
-        if ( $lo > 360 ) {
-            $lo -= 360;
-        }
- 
-        $lo /= $self->LoInc;
-        if ( $lo < 0 or $lo > $self->Ni ) {
-            return 0;
-        }
+    $lo += $self->{Lo_SHIFT};
+    if ( $lo > 360 ) {
+        $lo -= 360;
     }
-    else {
-        if ( $lo < $self->Lo1 or $lo > $self->Lo2 ) {
-            return 0;
-        }
+ 
+    $lo /= $self->LoInc;
+    if ( $lo < 0 or $lo > $self->Ni ) {
+        return 0;
     }
 
     return 1;
@@ -578,8 +557,30 @@ sub lalo2offset {
 
     
     my $thislong = 0;
+#    $long += $self->{Lo_SHIFT};
 
     $self->clearError;
+
+    # First check if values are out of range...
+    my ($lo1, $lo2, $la1, $la2) = ($self->Lo1, $self->Lo2, $self->La1, $self->La2, );
+    if ( $self->sn_scan_flag ) {
+        if ( $lat < $self->La1 or $lat > $self->La2 ) {
+            $self->error( "lalo2offset(): LAT >$lat< out of range $la1 to $la2" );
+            return -1;
+        }
+    }
+    else {
+        if ( $lat > $self->La1 or $lat < $self->La2 ) {
+            $self->error( "lalo2offset(): LAT >$lat< out of range $la1 to $la2" );
+            return -1;
+        }
+    }
+
+    if ( not $self->validLo( $long ) ) {
+        $self->error( "lalo2offset(): LONG: >$long< out of range $lo1 to $lo2" );
+        return -1;
+    }
+
     my $out;
     
     if ( $self->sn_scan_flag ) {
@@ -615,11 +616,8 @@ sub lalo2offset {
 # This will be much faster than repeated calls to extract() because only one
 # call to wgrib and just one file open are required.
 #
-# Returns a Geo::ReadGRIB::PlaceIterator object.
-#
-# Data is no longer stored in the current object by default as of version 
-# 1.4. To turn this behavior back on use the backflip() method;
-#
+# Returns a Geo::ReadGRIB::PlaceIterator object. All data extracted is also
+# stored in the current object.
 #
 # require: lat1 >= lat2 and long1 <= long2 - that is, lat1 is north or lat2
 #          and long1 is west of long2 (or is the same as)
@@ -635,10 +633,6 @@ sub extractLaLo {
     my $long2  = shift;
     my $time   = shift;
 
-    my $flipBack = $self->backflip() ? 1 : 0;
-    my $DEBUG = $self->{DEBUG} ? 1 : 0;
-    my $LoInc = $self->LoInc;
-
     my @types;
     if ( ref $type_s eq 'ARRAY' ) {
         push @types, @$type_s;
@@ -672,9 +666,21 @@ sub extractLaLo {
         return;
     }
 
-    if ( not $self->validLa($lat1) or not $self->validLa($lat2) ) {
-        $self->error( "extractLaLo(): LAT >$lat1 or $lat2< out of range $la1 to $la2" );
-        return;
+    if ( $self->sn_scan_flag ) {
+        if (   $lat1 < $self->La1 or $lat1 > $self->La2
+            or $lat2 < $self->La1 or $lat2 > $self->La2 )
+        {
+            $self->error( "extractLaLo(): LAT >$lat1 or $lat2< out of range $la1 to $la2" );
+            return;
+        }
+    }
+    else {
+        if (   $lat1 > $self->La1 or $lat1 < $self->La2
+            or $lat2 > $self->La1  or $lat2 < $self->La2 )
+        {
+            $self->error( "extractLaLo(): LAT >$lat1 or $lat2< out of range $la1 to $la2" );
+            return;
+        }
     }
 
     if ( not $self->validLo( $long1 ) or not $self->validLo( $long2 ) ) {
@@ -689,10 +695,9 @@ sub extractLaLo {
         $self->findNearestTime( $time );
     }
 
-    my ( $dtaLength, $fileDump, $offset, $dump, $record, $lo, $la );
+    my ( $offset, $dump, $record, $lo, $la );
     my $tm = $self->{THIS_TIME};
 
-
     for my $type ( @types ) {
         $record = $self->{catalog}->{$tm}->{$type};
 
@@ -702,68 +707,35 @@ sub extractLaLo {
         }
 
         my $tmp = $self->tempfile();
-        my $cmd = "\"$LIB_DIR\"/wgrib.exe \"$self->{fileName}\" -d $record -nh -o $tmp";
+        my $cmd =
+          "\"$LIB_DIR\"/wgrib.exe \"$self->{fileName}\" -d $record -nh -o $tmp";
         my $res = qx($cmd);
         my $F = IO::File->new( $tmp ) or croak "Can't open temp file";
 
-        # Make sure first offset is smallest for any scanning order 
-        my ( $offsetFst, $offsetLst ) = sort {$a <=> $b} ( 
-            $self->lalo2offset($lat1, $long1),  $self->lalo2offset($lat2, $long2) );
-        $dtaLength = ($offsetLst - $offsetFst +1) * 4;        
-        seek $F, $offsetFst * 4, 0;
-        read $F, $fileDump, $dtaLength; 
-
-        my ($x, $y);
         $dump = "";
-        for ( $la = $lat1, $x = 0 ; $la >= $lat2 ; $la -= $self->LaInc, $x++ ) {
-                my $num_lo = sprintf "%d", (($long2 - $long1) / $LoInc) +1;
-                $offset = $self->lalo2offset( $la, $long1 ) - $offsetFst;
-                my @luck = unpack "f*", substr $fileDump, $offset * 4, 4 * $num_lo;
-            for ( $lo = $long1, $y = 0 ; $lo <= $long2 ; $lo += $LoInc, $y++ ) {
-                my $dump = shift @luck;
-                $dump = defined $dump ? sprintf "%.2f", $dump : 0.00;
-                $dump = 0 if $dump eq '';
+        for ( $lo = $long1 ; $lo <= $long2 ; $lo += $self->LoInc ) {
+            for ( $la = $lat1 ; $la >= $lat2 ; $la -= $self->LaInc ) {
+                $offset = $self->lalo2offset( $la, $lo );
+#                $la = sprintf "%.4f", $la;
+#                $lo = sprintf "%.4f", $lo;
+                seek $F, $offset * 4, 0;
+                read $F, $dump, 4;
+                $dump = unpack "f", $dump;
+                $dump = sprintf "%.2f", $dump;
                 $dump = "UNDEF" if $dump > 999900000000000000000;
-                print gmtime($tm) . ": $self->{v_catalog}->{$type}  $dump\n" if $DEBUG;
-                $self->{data}->{$tm}->{$la}->{$lo}->{$type} = $dump if $flipBack;
-                # TODO look into the negative lats with CMS data 
-#               $self->{matrix}->{$tm}->{$type}->[$x]->[$y] = $dump if $la >= 0 and $lo >= 0;
+                print gmtime($tm) . ": $self->{v_catalog}->{$type}  $dump\n"
+                  if $self->{DEBUG};
+                $self->{data}->{$tm}->{$la}->{$lo}->{$type} = $dump;
                 $plit->addData( $tm, $la, $lo, $type, $dump );
             }
         }
         close $F;
         unlink $tmp;
-        undef $fileDump;
     }
 
     return $plit;
 }
 
-#--------------------------------------------------------------------------
-# backflip( [flag] ) - Getter/setter for the backflip() state.
-#
-# When true, certain older behaviors return. In version 1.4 this is only the
-# storage in the current object of data extracted by extractLaLo(). 
-#
-# default false 
-#--------------------------------------------------------------------------
-sub backflip {
-    my ( $self, $flag ) = @_;
-    $self->{backflip} = $flag if defined $flag;
-    return  $self->{backflip};
-}
-
-#--------------------------------------------------------------------------- 
-# getMatrix({ time => time, type => type }) 
-#
-# return the array of the extracted region for type and time 
-# POSSIBLE FUTURE FUNCTION
-#--------------------------------------------------------------------------- 
-#sub getMatrix {
-#    my	( $self, $r ) = @_;
-#    return $self->{matrix}->{ $r->{time} }->{ $r->{type} };
-#}
-
 #--------------------------------------------------------------------------
 # $plit = extract(data_type, lat, long, [time])
 #
@@ -783,22 +755,10 @@ sub extract {
     my $long = shift;
     my $time = shift;
 
-    # First see that requested values are in range...
-    my ($lo1, $lo2, $la1, $la2) = ($self->Lo1, $self->Lo2, $self->La1, $self->La2, );
-    if ( not $self->validLa($lat) ) {
-        $self->error( "extract(): LAT >$lat< out of range $la1 to $la2" );
-        return;
-    }
-
-    if ( not $self->validLo($long) ) {
-        $self->error( "extract(): LONG: >$long< out of range $lo1 to $lo2" );
-        return;
-    }    
-
     my $offset = $self->lalo2offset( $lat, $long );
 
     if ( $offset < 0 ) {
-        $self->error( "extract(): offset less than zero" );
+        # do not reset error, there will be one from lalo2offset()
         return 1;
     }
 
@@ -915,16 +875,6 @@ sub getDataHash {
     return $self->{data};
 }
 
-
-#--------------------------------------------------------------------------
-#  clearData( )
-#--------------------------------------------------------------------------
-sub clearData {
-    my $self = shift;
-    undef $self->{data};
-    return ;
-}
-
 #--------------------------------------------------------------------------
 # getError()
 #
@@ -1053,9 +1003,7 @@ sub show {
 
 1;
 
-__DATA__
-
-=pod
+__END__
 
 =head1 NAME
 
@@ -1102,11 +1050,7 @@ Binary" format Weather data files.
   $data = $w->getDataHash();
   
   # $data contains a hash reference to all grib data extracted
-  # by extract() the object in its lifetime.
-  #
-  # As of Version 1.35 extractLaLo() does not save extracted data to 
-  # the object by default and so it will not be returned by getDataHash().
-  # Use the backflip() method to bring back this older behavior.
+  # by the object in its lifetime.
   #  
   # $data->{time}->{lat}->{long}->{data_type} now contains data 
   # for data_type at lat,long and time unless there was an error
@@ -1165,22 +1109,22 @@ calling wgrib.exe
 
 =over 4
 
-=item $O = new Geo::ReadGRIB "grib_file";
+=item $object = new Geo::ReadGRIB "grib_file";
 
 Returns a Geo::ReadGRIB object that can open GRIB format file "grib_file".
 wgrib.exe is used to extract full header info from the "self describing" GRIB
 file. 
 
-=item $O->getFullCatalog();
+=item $object->getFullCatalog();
 
 getFullCatalog() will extract the full text descriptions of data items in the GRIB 
 file and store them in the object.
 
-=item $O->getParam("show");
+=item $object->getParam("show");
 
 I<getParam(show)> Returns a string listing the names of all published parameters.
 
-=item $O->getParam(param);
+=item $object->getParam(param);
 
 I<getParam(param)> returns a scalar with the value of param where param is one
 of TIME, LAST_TIME, La1, La2, LaInc, Lo1, Lo2, LoInc, fileName.
@@ -1219,39 +1163,37 @@ data.
 =back
 
 
-=item $O->show();
+=item $object->show();
 
 Returns a formatted text string description of the data in the GRIB file.
 This includes latitude, longitude, and time ranges, and data type text 
 descriptions (if getFullCatalog() has been run).
 
-=item $plit = $O->extractLaLo([data_type, ...], lat1, long1, lat2, long2, time);
+=item $plit = $object->extractLaLo([data_type, ...], lat1, long1, lat2, long2, time);
 
-Extracts forecast data for a given time and data type(s) for a range of 
+Extracts forecast data for a given type and time and data type(s) for a range of 
 locations. The locations will be all (lat, long) points in the GRIB file inside the 
 rectangular area defined by (lat1, long1) and (lat2, long2) where lat1 >= lat2
 and long1 <= long2. That is, lat1 is north or lat2 and long1 is west of long2
-(or equal to...)
+(or is the same as...)
 
-data_type is either a data type name as a string or a list of data name strings
+data_type is either a data type name as a string of a list of data name strings
 as an array reference. 
 
 Time will be in epoch seconds as returned, for example, by 
-Time::Local. If the time requested, is in the range of times in the file, but not 
+Time::Local. If the time requested is in the range of times in the file but not 
 one of the exact times in the file, the nearest existing time will be used. An 
 error will be set if time is out of range.
 
 Returns a L<Geo::ReadGRIB::PlaceIterator> object containing the extracted data
 which can be used to iterate through the data in order sorted by lat and then long.
 
-Extracted data is not retained in the ReadGRIB object data structure by default
-as of ReadGRIB version 1.4. Use the backflip() method to turn this behavior 
-back on.
+Extracted data is also retained in the ReadGRIB object data structure.
 
 Since extractLaLo() needs only one call to wgrib and one temp file open,
 this is faster than using extract() to get the same data points one at a time.
 
-=item $plit = $O->extract(data_type, lat, long, I<time>);
+=item $plit = $object->extract(data_type, lat, long, I<time>);
 
 Extracts forecast data for given type and location. I<time> is optional.
 Extracts data for all times in file unless a specific time is given 
@@ -1262,7 +1204,7 @@ long. If lat or long is out of range for the current file an error will be
 set ( see getError() ).
 
 time will be in epoch seconds as returned, for example, by Time::Local. If the 
-time requested is in the range of times in the file, but not one of the exact 
+time requested is in the range of times in the file but not one of the exact 
 times in the file, the nearest existing time will be used. An error will be set
 if time is out of range.
 
@@ -1273,122 +1215,37 @@ which can be used to iterate through the data in order sorted by lat and then lo
 
 Extracted data is also retained in the ReadGRIB object data structure.
 
-=item $O->getError();
+=item $object->getError();
 
 Returns string messages for the last error set. If no error is set getError()
 returns undef. Only the most recent error will be show. It's good practice to 
 check errors after actions, especially while developing applications.
 
-=item $bfFlag = $O->backflip();
-
-When true, certain older behaviors return. In version 1.4 this is only the
-storage in the current object of data extracted by extractLaLo().
-
-backflip() returns false by default
-
-=item $O->getDataHash();
+=item $object->getDataHash();
 
 Returns a hash ref with all the data items in the object. This will be all the 
 data extracted from the GRIB file for in the life of the object.
-
-As of Version 1.35 extractLaLo() does not save extracted data to the object by 
-default and so it will not be returned by getDataHash(). Use the backflip() 
-method to bring back this older behavior.
  
 The hash structure is
 
    $d->{time}->{lat}->{long}->{type}
 
-=back
+=item $object->getCatalog() DEPRECATED; 
 
-=head1 DEPRECATED METHODS
-
-=over
-
-=item $O->getCatalog() DEPRECATED; 
-
-=item $O->getCatalogVerbose() DEPRECATED;
+=item $object->getCatalogVerbose() DEPRECATED;
 
 getCatalog() is DEPRECATED and no longer does anything but set an error. 
 It's function of Getting the critical offset index for each data type and time 
 in the file is now done during object creation.
 
-getCatalogVerbose() is also DEPRECATED as redundant and now just calls 
+getCatalogVerbose() is also DEPRECATED as redundent and now just calls 
 getFullCatalog(). and sets an error.    
 
 =back
 
-=head1 PRIVATE METHODS
-
-The following  are not public methods and may be removed or changed in
-future releases. Do not use these in production code.
-
-=over
-
-=item La1()
-
-=item La2()
-
-=item Lo1()
-
-=item Lo2()
-
-=item LaInc()
-
-=item LoInc()
-
-=item Ni()
-
-=item Nj()
-
-=item TR()
-
-=item adjustLong()
-
-=item calInc()
-
-=item clearError()
-
-=item clearData()
-
-=item dumpit()
-
-=item error()
-
-=item findNearestTime()
-
-=item lalo2offset()
-
-=item m2ft()
-
-=item openGrib()
-
-=item parseGDS()
-
-=item sn_scan_flag()
-
-=item tempfile()
-
-=item toDecimal()
-
-=item validLo()
-
-=item validLa()
-
-=back
-
 =head1 BUGS AND LIMITATIONS
 
-There are no known bugs in this module version. Geo::ReadGRIB versions before
-1.1 are known to give results that are sometimes off by one LoInc west or 
-east, only on 64bit Perl where nvtype='long double'. Geo::ReadGRIB 1.1 and 
-above will not exhibit this bug. 
-
-Versions between 0.98_1 and 1.21 may not parse the time headers correctly for some
-forecast records in CMC grib files.  Geo::ReadGRIB 1.3 and above will not exhibit 
-this bug. 
-
-Please report problems through
+There are no known bugs in this module. Please report problems through
 
 http://rt.cpan.org
 
@@ -1419,6 +1276,3 @@ at your option, any later version of Perl 5 you may have available.
 
 
 =cut
-
-__C__
-
